//#include <Bounce2.h>                         // Bibliothek für Taste
#include <AccelStepper.h>                      // Stepper Bibliothek einbinden
// angepasst für Stepper 28BYJ-48
#include <IRremote.h>                          // Bibliothek für Infrarot-Fernbedienung

#define motorPin1  9                           // IN1 am Steppertreiber SL 1000
#define motorPin2  10                          // IN2
#define motorPin3  11                          // IN3
#define motorPin4  12                          // IN4
#define HALFSTEP 8                             // Halbschrittbetrieb, Acht Schritte pro Sequenz 

AccelStepper stepper1(HALFSTEP, motorPin1, motorPin3, motorPin2, motorPin4);
float istwert;

#define IR_Eingang_PIN 4                       // Pin Infrarot
#define Nullsensor_PIN 8                       // Pin Hallsensor
#define Relay_PIN 2                            // Pin Umpolungstrelais 

#define korrektur_schrittweite 10              // Anzahl Schritte zur Feinjustierung mit Nullpunktverscheibung (*/#)
#define kleine_schrittweite 5                  // Anzahl Schritte zur Feineinstellung (Pfeil links/rechts)
#define grosse_schrittweite 40                 // Anzahl Schritte zur Grobeinstellung (Pfeil oben unten)

typedef struct {                               // Werteliste als Struct
  uint8_t Position;
  unsigned long IR_Wert;
  int Winkel_Wert;
  bool Relais;
} Winkel;

const Winkel Winkel_Liste[] {                  // Werteliste füllen
  {0, 3860463360, 15, 0},                      //Taste 0 - IR-Wert, Winkel, Relais-an-aus
  {1, 3125149440, 45, 0},                      //Taste 1
  {2, 3108437760, 75, 0},                      //Taste 2
  {3, 3091726080, 105, 0},                     //Taste 3
  {4, 3141861120, 135, 0},                     //Taste 4
  {5, 3208707840, 165, 1},                     //Taste 5
  {6, 3158572800, 195, 1},                     //Taste 6
  {7, 4161273600, 225, 1},                     //Taste 7
  {8, 3927310080, 255, 1},                     //Taste 8
  {9, 4127850240, 285, 1},                     //Taste 9

  //Die weiteren Tasten sind für Zusatzfunktionen wie Grob- und Feinbewegung sowie Justierungbelegt und dürfen nicht verändert werden
  
  {10, 3910598400, -1, 0},                     //Taste Stern
  {11, 4061003520, -2, 0},                     //Taste Raute
  {12, 3877175040, -3, 0},                     //Taste oben
  {13, 4144561920, -4, 0},                     //Taste links
  {14, 3810328320, -5, 0},                     //Taste ok
  {15, 2774204160, -6, 0},                     //Taste rechts
  {16, 2907897600, -7, 0},                     //Taste unten
};

int digitalVal ; // digital readings

void setup()
{
  pinMode (IR_Eingang_PIN, INPUT);
  pinMode (Relay_PIN, OUTPUT);
  digitalWrite(Relay_PIN, 0);

  IrReceiver.begin(IR_Eingang_PIN, ENABLE_LED_FEEDBACK);

  Serial.begin(9600);                         // Wertekontrolle im Serialen Monitor
  Serial.println("Start");

  null_pinkt_setzen();                        // Funktin Nullpunkt suchen und setzen

}

void loop() {

  float sollwert;
  float strecke;

  if (IrReceiver.decode()) {
    for (uint8_t i = 0; i < sizeof(Winkel_Liste) / sizeof(Winkel); ++i) {

      if (Winkel_Liste[i].IR_Wert == IrReceiver.decodedIRData.decodedRawData) {

        int Tastenwert = Winkel_Liste[i].Winkel_Wert;

        if (Tastenwert < 0 )                  // Sonderfunktionen *, #. OK, Pfeiltasten
        {
          switch (Tastenwert)
          {
            case -1 : schritte_speichernd(korrektur_schrittweite * -1); break;
            case -2 : schritte_speichernd(korrektur_schrittweite); break;
            case -3 : schritte(grosse_schrittweite * -1); break;
            case -4 : schritte(kleine_schrittweite * -1); break;
            case -5 : null_pinkt_setzen(); break;
            case -6 : schritte(kleine_schrittweite); break;
            case -7 : schritte(grosse_schrittweite); break;
            default : break;

          }
        }
        else                                  // Positionierung über Zahlentasten
        {

          stepper1.stop();                    // Stepper anhalten

          // winkel 4096 / 360 * Winkelwert
          sollwert = (4096.0 / 360.0) * Tastenwert;

          Serial.println("-");                /// Protokollausgabe auf Seriellem Monitor
          Serial.print("Winkelwert: ");
          Serial.println(Tastenwert);
          Serial.print("istwert: ");
          Serial.println(istwert);
          Serial.print("sollwert: ");
          Serial.println(sollwert);

          if (istwert != sollwert) {
            strecke =  sollwert - istwert;
            Serial.print("Strecke: ");        // Protokollausgabe auf Seriellem Monitor
            Serial.println(strecke);          // Protokollausgabe auf Seriellem Monitor
            stepper1.move(strecke);
            stepper1.run();
          }
          istwert = sollwert;
          digitalWrite(Relay_PIN, Winkel_Liste[i].Relais);  // Relais schalten
        }
      }
    }
    IrReceiver.resume();
  }

  boolean isRunning = stepper1.run();
  if (!isRunning) stepper1.disableOutputs();
}



// Funktion Nullpunkt suchen und setzen
void null_pinkt_setzen()
{
  // Fahrt zur Nullstellung
  stepper1.setMaxSpeed(200.0);                 // Max Geschwindigkeit, Beschleunigung und gesetzte Geschwindigkeit einstellen
  stepper1.setAcceleration(200.0);             // Zum Suchen des Endschalters langsam fahren
  stepper1.setCurrentPosition(0);              // Position definiert auf "0" setzen
  stepper1.moveTo(-20000);                     // Einen weiten Weg vorgeben, damit die Drehscheibe den Nullpunkt überfährt

  digitalVal = digitalRead(Nullsensor_PIN) ;   // Hallsensor einlesen, damit in der while-Schleife ein definierter Zustand entsteht
  Serial.println ("Nullpunkt suchen");         // Protokollausgabe auf Seriellem Monitor
  while (digitalVal == 0) {                    // Diese Schleife durchlaufen, solange der Nullpunkt nicht erreicht ist.
    digitalVal = digitalRead(Nullsensor_PIN);  // Nullpunkt bei jedem Durchlauf testen
    stepper1.run ();                           // Stepper bewegen
  }

  // Endschalter wurde erreicht, while Schleife wird verlassen
  stepper1.stop();                             // Stepper anhalten
  delay (1000);                                // Warten, bis Stepper steht
  Serial.println ("Nullpunkt erreicht");       // Protokollausgabe auf Seriellem Monitor

  istwert = 0;                                 // Nullposition merken
  stepper1.setCurrentPosition(0);              // Position definiert auf "0" setzen

  stepper1.setSpeed(50);
  stepper1.setMaxSpeed(400);                   // Max Geschwindigkeit, Beschleunigung und gesetzte Geschwindigkeit einstellen
  stepper1.setAcceleration(300);
}


//Funktion Grob- und Feineinstellung per Pfeiltasten
void schritte(int schrittweite)
{
  Serial.println(schrittweite);
  stepper1.move(schrittweite);

  while (stepper1.run()) {}
  stepper1.disableOutputs();
  delay (1000);
}


//Funktion Feinjustierung unter Beibehaltung der Schrittposition (Nullpunktverscheibung)
void schritte_speichernd(int schrittweite1)
{
  long aktuelle_position = stepper1.currentPosition();
  Serial.println(schrittweite1);              // Protokollausgabe auf Seriellem Monitor
  stepper1.move(schrittweite1);

  while (stepper1.run()) {}
  stepper1.disableOutputs();
  delay (1000);
  stepper1.setCurrentPosition(aktuelle_position);
}
